// Title:   MLSApplication.cpp
// Created: Thu Sep 25 15:11:22 2003
// Authors: Richard Keiser, Mark Pauly, Sascha Scandella
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#include "MLSApplication.h"


void MLSApplication::run() { 

	/// mls projection
	int j;
	if (!valid) {
		return;
	}
	
	this->mls->projection();

	for (j = 0, this->surfel = this->surfels->getFirstSurfel(); j < this->numberOfSurfels; j++, this->surfel = this->surfels->getNextSurfel()) {
		scene->setSurfelPosition(surfel, this->newPositions[j], false);
		
		if (this->adjustNormals) {
			scene->setSurfelNormal(surfel, this->newNormals[j], false);
		}
	}

	if (mls) {
		delete this->mls;
		this->mls = NULL;
	}


	scene->emitObjectModifiedSignal();
	scene->emitRenderFrameSignal();

}


void MLSApplication::init() {

	// build neighbourhood and store positions and normals
	int j;

	this->scene = Scene::getInstance();
	this->object = scene->getActiveObject();
	
	if (object) {
		
		this->surfels = object->getSurfelCollection();
					
		this->numberOfSurfels = surfels->getNofSurfels();

		if ((2 * this->numberOfNeighbours) >= this->numberOfSurfels) {
			this->valid = false;
			return;
		}
		else {
			valid = true;
		}

		this->positions = new Vector3D [numberOfSurfels];
		this->normals = new Vector3D [numberOfSurfels];
		this->newPositions = new Vector3D [numberOfSurfels];
		this->newNormals = new Vector3D [numberOfSurfels];
		
		// create search structure for nearest point queries
		for (j = 0, surfel = surfels->getFirstSurfel(); j < surfels->getNofSurfels(); j++, surfel = surfels->getNextSurfel()) {
			positions[j] = surfel->getPosition();
			normals[j] = surfel->getNormal();
		}

		this->nbHood = new NeighbourHood (positions, this->numberOfSurfels);
		this->mls = new MovingLeastSquares(this->nbHood, this->numberOfSurfels, this->positions, this->normals, this->newPositions, this->newNormals,
										   this->numberOfNeighbours, this->polynomial, this->approximation, this->recalculateNormals, this->adjustNormals);
	}

}

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
