// Title:   VolGridParticleSim.h
// Created: Thu Sep 25 15:10:00 2003
// Authors: Richard Keiser, Mark Pauly, Sascha Scandella
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __VOLGRIDPARTICLESIMULATION_H_
#define __VOLGRIDPARTICLESIMULATION_H_

#include "VolGrid.h"

/**
 * VolGridParticleSimulation is used to generate an surface based on <i>Particle Simulation</i>.
 */
class VolGridParticleSimulation : public OcTreeIterator {

public:

	/**
	 * @param grid
	 *				reference to the volumetric grid.
	 * @param numSamples
	 *				number of surfels to generate.
	 */	
	VolGridParticleSimulation(VolGrid *grid, int numSamples) {
		this->grid = grid;
		showInternalNodes = false;
		this->numSamples = numSamples;
	}
	
	~VolGridParticleSimulation() { }

	/**
	 * Needed to iterate the octree.
	 */	
	virtual bool iterate(OcTreeTraversalNode *travNode);

	/**
	 * This method is called to run the <i>Particle Simulation</i>.
	 */	
	void build();
	
	/**
	 * This method is called to repel the particles.
	 */	
	void repel();

public:
	/// Reference to the volumetric grid.
	VolGrid			 *grid;
	// Reference to the 
	SurfelCollection *surfels;
	/// Reference to the scene
	Scene			 *scene;
	/// Reference to surfel positions.
	Vector3D		 *positions;	
	/// Reference to surfel normals.
	Vector3D		 *normals;
	/// Variation values
	float			 *variation;
	/// Influence of the variation.
	float			 variationInfluence;
	/// Average radius
	float			 avgRadius;
	/// Number of surfels
	int				 numSamples;
	/// Reference to spatial search structure (kd-tree)
	NeighbourHood	 *originalSearch;
	/// Average curvature factor.
	float			 variationAvg;
	/// Number of sampels per cell.
	float			 numSamplesPerCell;
	/// Residual value.
	float			 rest;
	/// number of cells.
	int				 cellCounter;
	///	Damping value
	float			 damping;
	/// Scaling constant.
	float			 forceScale;
	/// Number of iterations
	unsigned int	 numIterations;
	/// Number of neighbours
	unsigned int	 numNeighbours;
};

#endif	// __VOLGRIDPARTICLESIMULATION_H_
// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
