// Title:   VolGridReader.h
// Created: Thu Sep 25 15:09:55 2003
// Authors: Richard Keiser, Mark Pauly, Sascha Scandella
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __VOLGRIDREADER_H_
#define __VOLGRIDREADER_H_

#include "VolGrid.h"

// Note: Only for debugging purposes (code not optimized)
class VolGridReader {

public:

	VolGridReader() { }

	~VolGridReader() { }

	static inline void writeUnsignedShort(const void *p, size_t nofItems, FILE *fp);
	static inline void writeLong(const void *p, size_t nofItems, FILE *fp);
	static inline void writeFloat(const void *p, size_t nofItems, FILE *fp);
	static inline void writeInt(const void *p, size_t nofItems, FILE *fp);
	static inline void writeBool(const void *p, size_t nofItems, FILE *fp);
	static inline void readUnsignedShort(void *p, size_t nofItems, FILE *fp);
	static inline void readLong(void *p, size_t nofItems, FILE *fp);
	static inline void readFloat(void *p, size_t nofItems, FILE *fp);
	static inline void readInt(void *p, size_t nofItems, FILE *fp);
	static inline void readBool(void *p, size_t nofItems, FILE *fp);

	bool save(FILE *fp);
	void saveNode(VolGridNode *node);
	bool load(FILE *fp);
	void loadNode(VolGridNode *node);

public:
	FILE					*fp;
	VolGrid					*grid;
	std::set<float*>		distanceValuesSet;
	std::map<int, float*>	distanceValuesMap;
	std::map<int, float*>	diffusionValuesMap;
	int						distanceValueCounter;
};

#endif	// __VOLGRID_H_

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
